from sc2.ids.unit_typeid import UnitTypeId
from sc2.ids.ability_id import AbilityId
from sc2.constants import *
import random

class Resources():
    def __init__(self, bot=None):
        self.bot = bot
        self.maxworkers = 0
        self.bot.neededworkers = 0
        self.assigned_harvesters = 0

    async def manageSupply(self):
        # Supply
        if self.bot.supply_left < 7:
            supplyposition = self.bot.hq_location.random_on_distance(random.randrange(1, 12))
            await self.bot.unitmanager.createSupply(supplyposition)

        # Gas
        supplybuildingname = self.bot.techtree.get_supplyBuilding()
        supplybuildingid = self.bot.techtree.get_id_from_name(supplybuildingname)
        supplybuildingcount = self.bot.units(UnitTypeId(supplybuildingid)).amount

        if supplybuildingcount > 0:
            for hq in self.bot.units(self.bot.townhall).ready:
                vgs = self.bot.state.vespene_geyser.closer_than(15, hq.position)
                for vg in vgs:
                    if not self.bot.units(self.bot.gasbuilding).closer_than(1.0, vg).exists:
                        if self.bot.can_afford(self.bot.gasbuilding):
                            worker = self.bot.select_build_worker(vg.position)
                            if worker is None:
                                break
                            self.bot.microActions.append(worker.build(self.bot.gasbuilding, vg))


    async def manageWorkers(self):
        """Creates and assigns workers"""
        # Distribute idle workers
        randomhq = self.bot.units(self.bot.townhall).ready
        if randomhq:
            randomhqpos = randomhq.random.position
            for lazy in self.bot.workers.idle:
                if randomhqpos:
                    self.bot.microActions.append(lazy.move(randomhqpos, queue=True))
            for lazyathq in self.bot.workers.idle:
                if randomhqpos:
                    mineral = self.bot.state.mineral_field.closest_to(randomhqpos)
                    self.bot.microActions.append(lazyathq.gather(mineral, queue=True))
                else:
                    break

        # Assign workers to minerals
        self.assigned_harvesters = 0
        for hq in self.bot.units(self.bot.townhall).ready:
            self.assigned_harvesters = hq.assigned_harvesters + self.assigned_harvesters

            if hq.assigned_harvesters > hq.ideal_harvesters:
                toomuch = hq.assigned_harvesters - hq.ideal_harvesters
                harvesters = self.bot.units(self.bot.worker).ready
                harvesters_pool = []

                harvesters_pool.extend(harvesters.random_group_of(toomuch))
                if harvesters_pool:
                    for harvester in harvesters_pool:
                        for checkhq in self.bot.units(self.bot.townhall).ready:
                            if checkhq.assigned_harvesters < checkhq.ideal_harvesters:
                                mineral = self.bot.state.mineral_field.closest_to(checkhq)
                                self.bot.microActions.append(harvester.gather(mineral))

        # Assign workers to assimilators
        if self.assigned_harvesters >= 5:
            if self.bot.units(self.bot.gasbuilding).ready.exists:
                for gasstation in self.bot.units(self.bot.gasbuilding):
                    workers = self.bot.units(self.bot.worker)
                    assigned = gasstation.assigned_harvesters
                    ideal = gasstation.ideal_harvesters
                    needed = ideal - assigned

                    worker_pool = []
                    for x in range(0, needed):
                        worker_pool.extend(workers.random_group_of(min(needed, len(workers))))
                        if worker_pool:
                            w = worker_pool.pop()
                            if len(w.orders) == 1 and w.orders[0].ability.id in [AbilityId.HARVEST_RETURN]:
                                self.bot.microActions.append(w.move(gasstation))
                                self.bot.microActions.append(w.return_resource(queue=True))
                            else:
                                self.bot.microActions.append(w.gather(gasstation))

        # Build Workers
        self.bot.neededworkers = 0
        workerscount = self.bot.units(self.bot.worker).amount
        assimilators = self.bot.units(self.bot.gasbuilding)
        for a in assimilators:
            self.bot.neededworkers = self.bot.neededworkers + a.ideal_harvesters
            self.maxworkers = self.maxworkers + 3

        for h in self.bot.units(self.bot.townhall).ready:
            self.bot.neededworkers = self.bot.neededworkers + h.ideal_harvesters
            self.maxworkers = self.maxworkers + 16

        if (workerscount >= self.maxworkers) or (workerscount >= self.bot.neededworkers):
            return

        for hq in self.bot.units(self.bot.townhall).ready:
            if hq.is_idle and self.bot.can_afford(self.bot.worker):
                self.bot.microActions.append(hq.train(self.bot.worker))
                return
